## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
## 
## CvEventManager
## This class is passed an argsList from CvAppInterface.onEvent
## The argsList can contain anything from mouse location to key info
## The EVENTLIST that are being notified can be found

#import os

from CvPythonExtensions import *
#from PyHelpers import PyPlayer
#import CvUtil
#import Popup as PyPopup
#import CvScreensInterface

import CvAlertManager

import CvJFortZoCUtils

###################################################
import CvJDataStorageUtils as data

# data.initTypesDict(pUnit, dictInit=None)
# data.initTypeStatsDict(pUnit, xType)
# data.hasTypesDict(pUnit)
# data.hasTypeStatsDict(pUnit, xType)
# data.hasTypeStat(pUnit, xType, xStat)
# data.setTypeStatsDict(pUnit, xType, dictStats=None)
# data.setTypeStat(pUnit, xType, xStat, xStatValue=None)
# data.delTypesDict(pUnit)
# data.delTypeStatsDict(pUnit, xType)
# data.delTypeStat(pUnit, xType, xStat)
# data.getDataIDStr(pObj=None)
# data.getUnitStatsStrType_StrStatList(pUnit, strDelim="_", bStatsTypeFirst=True)
# data.getTypesDict(pUnit)
# data.getTypeStatsDict(pUnit, xType)
# data.getTypeStat(pUnit, xType, xStat)

# data.initGlobalDict()
# data.hasGlobalDict()
# data.hasGlobalData(xKey)
# data.setGlobalData(xKey, xValue)
# data.getGlobalData(xKey=None)

###################################################
# globals
gc = CyGlobalContext()
localText = CyTranslator()

alert = CvAlertManager.CvAlertManager()

forts = CvJFortZoCUtils.CvJFortZoCUtils()

###################################################
class CvJFortZoCEventManager:

	def __init__(self, eventManager):
		self.EventKeyDown=6
		self.eventManager = eventManager

		#eventManager.addEventHandler("mouseEvent", self.onMouseEvent)
		#eventManager.addEventHandler("kbdEvent", self.onKbdEvent)
		eventManager.addEventHandler("EndGameTurn", self.onEndGameTurn)
		eventManager.addEventHandler("improvementBuilt", self.onImprovementBuilt)
		#eventManager.addEventHandler("improvementDestroyed", self.onImprovementDestroyed)
		eventManager.addEventHandler("unitCreated", self.onUnitCreated)
		eventManager.addEventHandler("unitKilled", self.onUnitKilled)
		eventManager.addEventHandler("unitBuildImprovement", self.onUnitBuildImprovement)

	################################################################################################

	def onEndGameTurn(self, argsList):
		'Called at the end of the end of each turn'
		iGameTurn = argsList[0]
		#return None

		# Get the dictionary of Fort Data
		dFortData = forts.getAllFortData()

                # If there is no Fort Data just return None
		if (len(dFortData.keys()) == 0):
                        return None

		# To each of these methods we send the arguments of
                # playerID, unitID, unitX, unitY
                # The first item in the lCommanders[iCommanderLoop] list is
                # the turn the Commander was created and is there
                # as a means to sort the list and run the code on each
                # fort in the order it was created
		for tLoopPlot in dFortData:
                        iLoopX = tLoopPlot[0]
                        iLoopY = tLoopPlot[1]
                        lLoopData = dFortData[tLoopPlot]
                        forts.updatePlotCulture(lLoopData[1], lLoopData[2], iLoopX, iLoopY)
			forts.updateCultureBorders(lLoopData[1], lLoopData[2], iLoopX, iLoopY)

	################################################################################################

	def onImprovementBuilt(self, argsList):
		'Improvement Built'
		iImprovement, iX, iY = argsList
                #alert.alert(0, "onImprovementBuilt")
                # When the fort is built we clear the mark we put on the
                # workers building it and we init are fort code

		iImprovementFort = gc.getInfoTypeForString("IMPROVEMENT_FORT")

		if (iImprovement == iImprovementFort):
                        pPlot = CyMap().plot(iX, iY)
			# Look for Worker on this plot
			pUnit = None
			iWorkers = 0
			for iUnitLoop in range(pPlot.getNumUnits()):
				pUnitLoop = pPlot.getUnit(iUnitLoop)
				bBuildFort = data.getTypeStat(pUnitLoop, "UnitData", "bBuildingFort")
				if (bBuildFort):
                                        data.delTypeStat(pUnitLoop, "UnitData", "bBuildingFort")
                                        iWorkers += 1
                                        if (pUnit == None):
                                                pUnit = pUnitLoop
                        # Do this so multiple commanders arn't created as this event is run
                        # for each worker completing the same improvement
                        if (iWorkers >= 1):
                                #forts.initFort(pUnit.getOwner(), iX, iY)
                                iUnitCommander = gc.getInfoTypeForString("UNIT_COMMANDER")
                                pPlayer = gc.getPlayer(pUnit.getOwner())
                                pUnit = pPlayer.initUnit(iUnitCommander, iX, iY, UnitAITypes.UNITAI_ATTACK, DirectionTypes.NO_DIRECTION)

        # This is not used
	def onImprovementDestroyed(self, argsList):
		'Improvement Destroyed'
		iImprovement, iOwner, iX, iY = argsList

	################################################################################################
		
	def onUnitCreated(self, argsList):
		'Unit Completed'
		pUnit = argsList[0]
		iUnitCommander = gc.getInfoTypeForString("UNIT_COMMANDER")
		if (pUnit.getUnitType() == iUnitCommander):
                        iPlayer = pUnit.getOwner()
                        iUnitID = pUnit.getID()
                        iX = pUnit.getX()
                        iY = pUnit.getY()
                        pUnit.setLevel(1)
                        forts.addFortData(CyGame().getGameTurn(), iPlayer, iUnitID, iX, iY)
                        forts.addFortZoCData(iPlayer, iX, iY)
                        forts.updateCultureBorders(iPlayer, iUnitID, iX, iY)
	
	def onUnitBuildImprovement(self, argsList):
		'Unit begins enacting a Build (building an Improvement or Route)'
		pUnit, iBuild, bFinished = argsList
                #alert.alert(0, "onUnitBuildImprovement")
		# When a worker starts building a fort we have to mark it
		# so we know who it is when its done building
		iBuildFort = gc.getInfoTypeForString("BUILD_FORT")
        	if (iBuild == iBuildFort):
                        data.setTypeStat(pUnit, "UnitData", "bBuildingFort", True)
	
	def onUnitKilled(self, argsList):
		'Unit Killed'
		pUnit, iAttacker = argsList

                # Just for fun destroy the fort when the Commander dies and
                # put city ruins in its place
                # XML Change: Forts no longer able to be pillaged
                # ToDo: Make a Fort Ruins improvement so we can do things with it
		iUnitCommander = gc.getInfoTypeForString("UNIT_COMMANDER")
		if (pUnit.getUnitType() == iUnitCommander):
                        forts.removeFortData(iX, iY)
                        pPlot = pUnit.plot()
                        iImprovementFort = gc.getInfoTypeForString("IMPROVEMENT_FORT")
                        if (pPlot.getImprovementType() == iImprovementFort):
                                pPlot.setImprovementType(gc.getInfoTypeForString("IMPROVEMENT_CITY_RUINS"))

	################################################################################################






















